package ink.wgink.gateway.component;

import ink.wgink.gateway.dao.route.IRouteDao;
import ink.wgink.gateway.filter.local.CommonFiltersDefinition;
import ink.wgink.gateway.pojo.route.Route;
import ink.wgink.gateway.util.UUIDUtil;
import org.springframework.cloud.gateway.filter.FilterDefinition;
import org.springframework.cloud.gateway.handler.predicate.PredicateDefinition;
import org.springframework.cloud.gateway.handler.predicate.RoutePredicateFactory;
import org.springframework.cloud.gateway.route.RouteDefinition;
import org.springframework.cloud.gateway.route.RouteDefinitionRepository;
import org.springframework.stereotype.Component;
import org.springframework.web.util.UriComponentsBuilder;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: RouteDefinitionRepository
 * @Description: 路由定义
 * @Author: wanggeng
 * @Date: 2021/4/22 4:04 下午
 * @Version: 1.0
 */
@Component
public class MongoRouteDefinitionRepository implements RouteDefinitionRepository {

    public IRouteDao routeDao;

    public MongoRouteDefinitionRepository(IRouteDao routeDao) {
        this.routeDao = routeDao;
    }

    @Override
    public Flux<RouteDefinition> getRouteDefinitions() {
        return this.routeDefinitionFlux();
    }

    @Override
    public Mono<Void> save(Mono<RouteDefinition> route) {
        return null;
    }

    @Override
    public Mono<Void> delete(Mono<String> routeId) {
        return null;
    }

    /**
     * 路由定义
     *
     * @return
     */
    private Flux<RouteDefinition> routeDefinitionFlux() {
        return routeDao.findAll().flatMap(route -> {
            // 定义路由
            RouteDefinition routeDefinition = new RouteDefinition();
            routeDefinition.setId(UUIDUtil.getUUID());
            routeDefinition.setUri(UriComponentsBuilder.fromUriString(route.getUri()).build().toUri());
            routeDefinition.setPredicates(this.listPredicateDefinition(route));
            // 过滤器
            routeDefinition.setFilters(this.listFilterDefinition(route));
            return Flux.just(routeDefinition);
        });
    }

    /**
     * 路由断言列表
     *
     * @param route
     * @return
     */
    private List<PredicateDefinition> listPredicateDefinition(Route route) {
        // 定义断言
        List<PredicateDefinition> predicateDefinitions = new ArrayList<>();
        // 路由断言
        PredicateDefinition pathPredicateDefinition = new PredicateDefinition();
        Map<String, String> pathPredicateDefinitionMap = new HashMap<>(2);
        pathPredicateDefinitionMap.put(RoutePredicateFactory.PATTERN_KEY, route.getPath());
        pathPredicateDefinition.setName("Path");
        pathPredicateDefinition.setArgs(pathPredicateDefinitionMap);
        // 添加路由断言
        predicateDefinitions.add(pathPredicateDefinition);
        return predicateDefinitions;
    }

    /**
     * 过滤器列表
     *
     * @return
     */
    private List<FilterDefinition> listFilterDefinition(Route route) {
        CommonFiltersDefinition commonFiltersDefinition = new CommonFiltersDefinition();
        // 定义过滤器
        List<FilterDefinition> filterDefinitions = new ArrayList<>();
        // 重写响应头中的location解决重定向问题
        filterDefinitions.add(commonFiltersDefinition.rewriteLocationResponseHeader(route));
        return filterDefinitions;
    }

}
